#coding: utf-8
# +-------------------------------------------------------------------
# | 宝塔Linux面板
# +-------------------------------------------------------------------
# | Copyright (c) 2015-2099 宝塔软件(http://bt.cn) All rights reserved.
# +-------------------------------------------------------------------
# | Author: hwliang <hwl@bt.cn>
# +-------------------------------------------------------------------

#+--------------------------------------------------------------------
#|   插件和模块加载器
#+--------------------------------------------------------------------

import os
import public
import json

def plugin_run(plugin_name,def_name,args):
    '''
        @name 执行插件方法
        @param plugin_name<string> 插件名称
        @param def_name<string> 方法名称
        @param args<dict_obj> 参数对像
        @return mixed
    '''
    # 获取插件目录
    plugin_path = public.get_plugin_path(plugin_name)

    # 检查插件目录是否合法
    plugin_file = os.path.join(plugin_path,plugin_name + '_main.py')
    if not public.path_safe_check(plugin_file): return public.returnMsg(False,'插件路径不合法')

    # 检查插件入口文件是否存在
    if not os.path.exists(plugin_file): return public.returnMsg(False,'指定插件入口文件不存在')

    # 添加插件目录到系统路径
    public.sys_path_append(plugin_path)

    try:
        # 引用插件入口文件
        _name = "{}_main".format(plugin_name)
        plugin_main = __import__(_name)

        # 检查类名是否符合规范
        if not hasattr(plugin_main,_name):
            return public.returnMsg(False,'指定插件入口文件不符合规范')
        
        # 实例化插件类
        plugin_obj = getattr(plugin_main,_name)()

        # 检查方法是否存在
        if hasattr(plugin_obj,def_name):
            # 执行方法
            return getattr(plugin_obj,def_name)(args)
        else:
            return public.returnMsg(False,'指定方法不存在')
    except SyntaxError as ex:
        return public.returnMsg(False,'指定插件不兼容当前操作系统')
    except Exception as ex:
        public.print_error()
        return public.returnMsg(False,'指定插件不存在')
    

def get_module_list():
    '''
        @name 获取模块列表
        @return list    
    '''

    module_list = []
    class_path = public.get_class_path()
    for name in os.listdir(class_path):
        path = os.path.join(class_path,name)
        # 过滤无效文件
        if not name:continue
        if name.endswith('.py'): continue
        if name[0] == '.': continue
        if not name.endswith('Model'): continue
        if os.path.isfile(path): continue

        module_list.append(name)
    
    return module_list

def module_run(module_name,def_name,args):
    '''
        @name 执行模块方法
        @param module_name<string> 模块名称
        @param def_name<string> 方法名称
        @param args<dict_obj> 参数对像
        @return mixed
    '''
    model_index = args.get('model_index',None)
    module_list = get_module_list()
    class_path = public.get_class_path()
    panel_path = public.get_panel_path()
    
    module_file = None
    if not model_index:
        # 如果没指定模块名称，则遍历所有模块目录
        for name in module_list:
            module_file = os.path.join(class_path,name,module_name + 'Model.py')
            if os.path.exists(module_file):
                _name = "{}Model".format(module_name)
                break
    else:
        # 新模块目录
        if model_index in ['mod']:
            _name = "{}Mod".format(module_name)
            module_file = os.path.join(panel_path,'mod','project',module_name + 'Mod.py')
        else:
            # 旧模块目录
            _name = "{}Model".format(module_name)
            module_file = os.path.join(class_path,model_index+"Model",module_name + 'Model.py')
        
    
    # 判断模块入口文件是否存在
    if not os.path.exists(module_file):
        return public.returnMsg(False,'没有在模型中找到指定模块')
    
    # 判断模块路径是否合法
    if not public.path_safe_check(module_file):
        return public.returnMsg(False,'模块路径不合法')
    
    public.sys_path_append(os.path.dirname(module_file))
    try:
        # 引用模块入口文件
        
        module_main = __import__(_name)

        # 检查模块是否符合规范
        if not hasattr(module_main,'main'):
            return public.returnMsg(False,'指定模块入口文件不符合规范')

        # 实例化模块类
        module_obj = getattr(module_main,'main')()

        # 检查方法是否存在
        if hasattr(module_obj,def_name):
            # 执行方法
            return getattr(module_obj,def_name)(args)
        else:
            return public.returnMsg(False,'指定方法不存在')
    except SyntaxError as ex:
        return public.returnMsg(False,'指定模块不兼容当前操作系统')
    except Exception as ex:
        public.print_error()
        return public.returnMsg(False,'指定模块不存在')
    

def get_plugin_list(upgrade_force = False):
    '''
        @name 获取插件列表
        @param upgrade_force<bool> 是否强制重新获取列表
        @return dict
    '''

    panel_path = public.get_panel_path()
    data_path = os.path.join(panel_path,'data')

    if not os.path.exists(data_path):
        os.makedirs(data_path,384)

    plugin_list = {}
    plugin_list_file = os.path.join(data_path,'plugin_list.json')
    if os.path.exists(plugin_list_file) and not upgrade_force:
        plugin_list_body = public.readFile(plugin_list_file)
        try:
            plugin_list = json.loads(plugin_list_body)
        except:
            plugin_list = {}
    
    if not os.path.exists(plugin_list_file) or upgrade_force or not plugin_list:
        pdata = public.get_user_info()
        url = public.GetConfigValue('home') + '/api/panel/get_soft_list_test'
        res = public.HttpPost(url,pdata)
        if not res: return public.returnMsg(False,'获取插件列表失败!')

        try:
            plugin_list = json.loads(res)
            public.writeFile(plugin_list_file,json.dumps(plugin_list))
        except:
            return public.returnMsg(False,'解析插件列表失败!')
        
    # 处理不兼容的插件
    for i in plugin_list['list']:
        if i['pid'] and i['type'] in [8,12]:
            i['ps'] = "<span style='color:red;'>[不兼容]</span>" + i['ps']
            i['soft'] = 9999

        
    return plugin_list


def start_total():
    '''
        @name 启动统计服务
        @return dict
    '''
    # 缺省状态不启用
    pass

def get_soft_list(args):
    '''
        @name 获取软件列表
        @param args<dict_obj> 参数对像
        @return dict
    '''
    pass


def db_encrypt(data):
    '''
        @name 数据库加密
        @param args<dict_obj> 参数对像
        @return dict
    '''
    # 缺省状态不启用
    return data

def db_decrypt(data):
    '''
        @name 数据库解密
        @param args<dict_obj> 参数对像
        @return dict
    '''
    # 缺省状态不启用
    return data

def plugin_end():
    '''
        @name 插件到期处理
        @return dict
    '''
    # 缺省状态不启用
    pass


def daemon_task():
    '''
        @name 后台任务守护
        @return dict
    '''
    # 缺省状态不启用
    pass

def daemon_panel():
    '''
        @name 面板守护
        @return dict
    '''
    # 缺省状态不启用
    pass

def flush_auth_key():
    '''
        @name 刷新授权密钥
        @return dict
    '''
    # 缺省状态不启用
    pass

def get_auth_state():
    '''
        @name 获取授权状态
        @return dict
    '''
    # 缺省状态不启用
    return public.returnMsg(False,'未授权')

    

        
    
            
    
    
    



    



    